#!/usr/bin/env python

"""
DOM macros for libxml2.
"""

__version__ = "0.1.2"

import xml.dom
import libxml2
import sys

# NOTE: libxml2 seems to use UTF-8 throughout.

def from_unicode(s):
    if type(s) == type(u""):
        return s.encode("utf-8")
    else:
        return s

def to_unicode(s):
    if type(s) == type(""):
        return unicode(s, encoding="utf-8")
    else:
        return s

_nodeTypes = {
    "attribute" : xml.dom.Node.ATTRIBUTE_NODE,
    "comment" : xml.dom.Node.COMMENT_NODE,
    "document_xml" : xml.dom.Node.DOCUMENT_NODE,
    "doctype" : xml.dom.Node.DOCUMENT_TYPE_NODE,
    "dtd" : xml.dom.Node.DOCUMENT_TYPE_NODE, # NOTE: Needs verifying.
    "element" : xml.dom.Node.ELEMENT_NODE,
    "entity" : xml.dom.Node.ENTITY_NODE,
    "entity_ref" : xml.dom.Node.ENTITY_REFERENCE_NODE,
    "notation" : xml.dom.Node.NOTATION_NODE,
    "pi" : xml.dom.Node.PROCESSING_INSTRUCTION_NODE,
    "text" : xml.dom.Node.TEXT_NODE
    }

def Node_nodeType(node):
    global _nodesTypes
    return _nodeTypes[node.type]

def Node_childNodes(node):

    # NOTE: Consider a generator instead.

    child_nodes = []
    node = node.children
    while node is not None:
        child_nodes.append(node)
        node = node.next
    return child_nodes

def _getNs(node):

    "Internal namespace information retrieval."

    try:
        return node.ns()
    except libxml2.treeError:
        return None

def Node_namespaceURI(node):
    ns = _getNs(node)
    if ns is not None:
        return to_unicode(ns.content)
    else:
        return None

def Node_nodeValue(node):
    return to_unicode(node.content)

def Node_prefix(node):
    ns = _getNs(node)
    if ns is not None:
        return to_unicode(ns.name)
    else:
        return None

def Node_nodeName(node):
    prefix = Node_prefix(node)
    if prefix is not None:
        return prefix + ":" + Node_localName(node)
    else:
        return Node_localName(node)

def Node_tagName(node):
    if node.type == "element":
        return Node_nodeName(node)
    else:
        return None

def Node_localName(node):
    return to_unicode(node.name)

def Node_parentNode(node):
    if node.type == "document_xml":
        return None
    else:
        return node.parent

def Node_previousSibling(node):
    if node.prev is not None:
        return node.prev
    else:
        return None

def Node_nextSibling(node):
    if node.next is not None:
        return node.next
    else:
        return None

# vim: tabstop=4 expandtab shiftwidth=4
